;** BASE_BOOTPROM.ASM  80MHZ VERSION V3.0b ***********************************
;   FOR BASE DSP PCB v3 [SINGLE FIBER VERSION] VER=3
;   ROUTINE TO PROGRAM SSI AND FETCH S-RECORDS -- BOOTS FROM EEPROM
;   COMPILE WITH  ASM56300 -A -B -V  filename
;                 SREC -B -A3 -OP:2 filename
;   then add      S20A000000003000000000C5 as 2nd line of .s file
;   then load into eeprom
;*****************************************************************************
    PAGE    110,60,1,1
    TABS    4
;*****************************************************************************
;   PERIPHERAL DEFINITIONS & POINTERS
;*****************************************************************************
START       EQU     $000F00                 ; program start location
INIT        EQU     $000200                 ; program initialization
BOOT        EQU     $000300                 ; program bootloader
SERIAL      EQU     $000000                 ; serial number location

TXREG       EQU     $FFFF89                 ; Transmit Data Register
RXREG       EQU     $FFFF8A                 ; Receive Data register
SEQREG      EQU     $FFFF88                 ; Sequence register
TXD         EQU     $FFFFBC                 ; ESSI0 Transmit Data Register
RXD         EQU     $FFFFB8                 ; ESSI0 Receive Data Register
SSISR       EQU     $FFFFB7                 ; ESSI0 Status Register
CRB         EQU     $FFFFB6                 ; ESSI0 Control Register B
CRA         EQU     $FFFFB5                 ; ESSI0 Control Register A
PCRC        EQU     $FFFFBF                 ; Port C (ESSI_0) control register
PCTL        EQU     $FFFFFD                 ; PLL control register
PDRB        EQU     $FFFFC9                 ; Port B (HOST) GPIO data
PRRB        EQU     $FFFFC8                 ; Port B (HOST) GPIO direction
PCRB        EQU     $FFFFC4                 ; Port B (HOST) control register
AAR3        EQU     $FFFFF6                 ; Address attribute register 3
BCR         EQU     $FFFFFB                 ; Bus control register

; The received 16-bit words are arranged in the following order:
; [0] command word | complement
; [1] number of 32-bit data words
; [2] memory space code X=1 Y=2 P=4 END=8
; [3] start address (bits 15..0)
; [ ] data words [merge two 16-bit words to make one 24-bit word]

;*******************************************************************************
;   INITIALIZE INTERRUPT VECTORS
;*******************************************************************************
            ORG     P:$0
            JMP     INIT

;*******************************************************************************
;   MAIN PROGRAM -- INITIALIZE AND READ SERIAL NUMBER
;*******************************************************************************
            ORG     P:INIT
PLL_SETUP   MOVEP   #>$078007,X:PCTL        ; set PLL parameters
EXT_SETUP   MOVEP   #>$FFFC21,X:AAR3        ; PERIPH $FFF000--$FFFFFF
            MOVEP   #>$073FE1,X:BCR         ; Set periph wait states
            MOVEP   #>3,Y:<<SEQREG          ; Set seqreg safe state
SSI_SETUP   MOVEP   #>$032070,X:CRB         ; async, LSB, enable TE RE
            MOVEP   #>$140803,X:CRA         ; 10 Mbps, 16 bit word
            MOVEP   #>$3F,X:PCRC            ; enable ESSI
PORTB_SETUP MOVEP   #>$1,X:PCRB             ; set PB[15..0] GPIO
            MOVEP   #>$FE,X:PRRB            ; TM GAIN .. OPT3V
            MOVEP   #>$0,X:PDRB             ;

REBOOT      MOVE    #BOOT,R0                ; relocate boot loader(128 words)
            MOVE    #START,R1               ; new start location
            DO      #$80,END_RELOC          ;
            MOVEM   P:(R0)+,A0              ;
            NOP                             ;
            MOVEM   A0,P:(R1)+              ;
            NOP                             ;
END_RELOC   NOP                             ;

W1_READ     MOVE    #SERIAL,R3              ; DS2401 1-wire read routine
W1_RESET    BCLR    #$0,X:PDRB              ; set PB0=0
            BSET    #$0,X:PRRB              ; output PE2=0            
            MOVE    #>$9600,X0              ; wait $9600=480us
            REP     X0                      ;
            NOP                             ;
            BCLR    #$0,X:PRRB              ; set PB0 input
            MOVE    #>$15E0,X0              ; wait $15E0=70us
            REP     X0                      ;
            NOP                             ;
            MOVEP   X:PDRB,X1               ; sample PB0, get ACK
            MOVE    #>$8020,X0              ; wait $8020=410us
            REP     X0                      ;
            NOP                             ; 
W1_WR       MOVE    #$33,A1                 ; data to xmit
            DO      #$8,END_WR              ; set up 8 clocks
WCLKS       BCLR    #$0,X:PDRB              ; set PB0=0
            BSET    #$0,X:PRRB              ; output PB0=0
            MOVE    #>$1E0,X0               ; wait $1E0=6us
            REP     X0                      ;
            NOP                             ;
WD0         JSET    #$0,A1,WD1              ;
            BSET    #$0,X:PRRB              ; output PB0=0
            MOVE    #>$10E0,X0              ; wait $10E0=54us
            REP     X0                      ;
            NOP                             ; 
            BCLR    #$0,X:PRRB              ; PB0 hi-Z
WD1         JCLR    #$0,A1,WD2              ;
            BCLR    #$0,X:PRRB              ; PB0=hi-Z
            MOVE    #>$10E0,X0              ; wait $10E0=54us
            REP     X0                      ;
            NOP                             ; 
WD2         BCLR    #$0,X:PRRB              ; PB0 hi-Z
            MOVE    #>$320,X0               ; wait $320=10us
            REP     X0                      ;
            NOP                             ;
            ASR     A                       ;
END_WR      NOP                             ;
W1_RD       DO      #$4,END_RD0             ;
RD0         CLR     A                       ;
RD1         DO      #$10,END_RD1            ; set up 16 clocks
            BCLR    #$0,X:PDRB              ; set PB0=0
            BSET    #$0,X:PRRB              ; output PB0=0
            MOVE    #>$1E0,X0               ; wait $1E0=6us
            REP     X0                      ;
            NOP                             ; 
            BCLR    #$0,X:PRRB              ; PB0 hi-Z
            MOVE    #>$2D0,X0               ; wait $2D0=9us
            REP     X0                      ;
            NOP                             ;
            MOVEP   X:PDRB,X1               ; sample PB0
RB0         JSET    #$0,X1,RB1              ;
            BCLR    #$0,A1                  ; bit=0
RB1         JCLR    #$0,X1,RB2              ;
            BSET    #$0,A1                  ; bit=1
RB2         NOP                             ;
            ASR     A                       ;
            MOVE    #>$1130,X0              ; wait $1130=55us
            REP     X0                      ;
            NOP                             ;
END_RD1     NOP                             ;
            ASR     #$8,A,A                 ;
            NOP
            MOVE    A0,X:(R3)+              ;
END_RD0     NOP                             ;
            BCLR    #$0,X:PRRB              ; set PB0 hi-Z
            JMP     START                   ;

;*******************************************************************************
;   RELOCATED PROGRAM CODE
;*******************************************************************************
            ORG     P:START,P:BOOT          ; P:runtime, P:loadtime
MAIN        JSR     READ16                  ;
            JSR     WRITE16                 ; 
            MOVE    A1,Y0                   ;

GET_SN      MOVE    #>$FF00,A               ; get DS2401 serial number
            CMP     Y0,A                    ;
            JNE     GET_CODE                ;

            MOVE    X:(SERIAL+0),A          ; send 4 x 16-bit 
            JSR     WRITE16                 ; serial number
            MOVE    X:(SERIAL+1),A          ;
            JSR     WRITE16                 ;
            MOVE    X:(SERIAL+2),A          ;
            JSR     WRITE16                 ;
            MOVE    X:(SERIAL+3),A          ;
            JSR     WRITE16                 ;

GET_CODE    MOVE    #>$FE01,A               ; load program code
            CMP     Y0,A                    ;
            JNE     MAIN                    ;

            JSR     READ16                  ; get NWORDS
            JSR     WRITE16                 ; 
            MOVE    A1,X1                   ; save word count
            JSR     READ16                  ; get MEMSPACE -- X,Y,P,END
            JSR     WRITE16                 ; 
            MOVE    A1,X0                   ; save mem space/word count
            JSR     READ16                  ; get ADDRESS
            JSR     WRITE16                 ; 
            MOVE    A1,R0                   ; save start address
            JSET    #$3,X0,FINISH           ; found END record
GET_WORDS   DO      X1,END_LOAD             ;
            JSR     READ16                  ; get 16-bit word
            JSR     WRITE16                 ; 
            ASR     #$10,A,A                ; shift right 16 bits
            JSR     READ16                  ;
            JSR     WRITE16                 ; 
            ASR     #$8,A,A                 ; 24-bit word in A0
X_LOAD      JCLR    #$0,X0,Y_LOAD           ; check X flag
            MOVE    A0,X:(R0)+              ;
Y_LOAD      JCLR    #$1,X0,P_LOAD           ; check Y flag
            MOVE    A0,Y:(R0)+              ;
P_LOAD      JCLR    #$2,X0,NO_LOAD          ; check P flag
            MOVEM   A0,P:(R0)+              ;
NO_LOAD     NOP                             ;
END_LOAD    NOP                             ;
            JMP     MAIN                    ; go get next S-record

    IF (VER==3)                             ; DSP_v3 pcb
READ16      JCLR    #$8,X:PDRB,RD16B        ; check RS485 or fiber
            JCLR    #$A,X:PDRB,*            ; wait for data in RXREG
            MOVE    Y:RXREG,A1              ; bits 15..0
            AND     #>$FFFF,A               ;
RD16B       JSET    #$8,X:PDRB,ENDRD16      ; check RS485 or fiber
            JCLR    #7,X:SSISR,*            ; wait for RDRF to go high
            MOVE    X:RXD,A1                ; read from ESSI
            NOP                             ;
ENDRD16     RTS                             ; 16-bit word in A1 (was RTS)
    ENDIF

WRITE16     JCLR    #$8,X:PDRB,WR16B        ; check RS485 or fiber
            MOVE    A1,Y:TXREG              ; write bits 15..0
WR16B       JSET    #$8,X:PDRB,ENDWR16      ;
            JCLR    #6,X:SSISR,*            ; wait for TDE
            MOVE    A1,X:TXD                ;
ENDWR16     RTS                             ;

FINISH      JMP     $0                      ; begin executing loaded program
                                            ; was (R0)
