;** GCAM_BOOTPROM.ASM V2.3 and V3.3 ******************************************
;   FOR GCAM DSP PCB v3 [SINGLE FIBER VERSION] VER=3
;   FOR GCAM DSP v1 & v2 [RS-485 VERSION] VER=2
;   ROUTINE TO PROGRAM SSI AND FETCH S-RECORDS -- BOOTS FROM EEPROM
;   COMPILE WITH  ASM56300 -A -B -V  filename
;                 SREC -B -A3 -OP:2 filename
;   then add      S20A000000003000000000C5 as 2nd line of .s file
;   then load into eeprom
;*****************************************************************************
    PAGE    110,60,1,1
    TABS    4
;*****************************************************************************
;   DEFINITIONS & POINTERS
;*****************************************************************************
START       EQU     $000FC0                 ; program start location
INIT        EQU     $000200                 ; program initialization
BOOT        EQU     $000300                 ; program bootloader
TXREG       EQU     $FFFF85                 ; Transmit Data Register
RXREG       EQU     $FFFF86                 ; Receive Data register
PDRB        EQU     $FFFFC9                 ; Port B (HOST) GPIO data
PRRB        EQU     $FFFFC8                 ; Port B (HOST) GPIO direction
PCRB        EQU     $FFFFC4                 ; Port B (HOST) control register
PCRC        EQU     $FFFFBF                 ; Port C (ESSI0) control register
PCRD        EQU     $FFFFAF                 ; Port D (ESSI1) control register
PRRD        EQU     $FFFFAE                 ; Port D (ESSI1) direction
PDRD        EQU     $FFFFAD                 ; Port D (ESSI1) data
TXD         EQU     $FFFFBC                 ; ESSI0 Transmit Data Register
RXD         EQU     $FFFFB8                 ; ESSI0 Receive Data Register
SSISR       EQU     $FFFFB7                 ; ESSI0 Status Register
CRB         EQU     $FFFFB6                 ; ESSI0 Control Register B
CRA         EQU     $FFFFB5                 ; ESSI0 Control Register A
PCTL        EQU     $FFFFFD                 ; PLL control register
AAR3        EQU     $FFFFF6                 ; Address attribute register 3
BCR         EQU     $FFFFFB                 ; Bus control register
SEQREG      EQU     $FFFF80                 ; external CCD SEQ register

; RS485 link -- the received 16-bit words are arranged:
; [0] number of words (exluding first two header words)
; [1] memory space code X=1 Y=2 P=4 END=8
; [2] start address (bits 15..0)
; [3] start address (bits 23..16), or coded as zero
; [ ] data words [merge two 16-bit words to make one 24-bit word]

; Optical link -- the received 16-bit words are arranged:
; [0] command word | complement
; [1] number of 32-bit data words
; [2] memory space code X=1 Y=2 P=4 END=8
; [3] start address (bits 15..0)
; [ ] data words [merge two 16-bit words to make one 24-bit word]

            ORG     P:$0
            JMP     INIT

            ORG     P:INIT
PLL_SETUP   MOVEP   #>$078007,X:PCTL        ; set PLL parameters
EXT_SETUP   MOVEP   #>$FFFC21,X:AAR3        ; PERIPH $FFF000--$FFFFFF
            MOVEP   #>$073FE1,X:BCR         ; Set periph wait states
            MOVEP   #>3,Y:<<SEQREG          ; Set seqreg safe state

PORT_SETUP  MOVEP   #>$1,X:PCRB             ; set PB[15..0] GPIO
            MOVEP   #>$0,X:PCRC             ; set PC[5..0] GPIO
            MOVEP   #>$0,X:PCRD             ; set PD[5..0] GPIO
            MOVEP   #>$F,X:PRRD             ; TM GAIN /ENRX /ENTX RXRDY
            MOVEP   #$0,X:PDRD              ; enable RS485
    IF (VER==2)
            MOVEP   #>$032070,X:CRB         ; async, LSB, enable TE RE
            MOVEP   #>$140803,X:CRA         ; 10 Mbps, 16 bit word
            MOVEP   #>$3F,X:PCRC            ; enable ESSI
    ENDIF

REBOOT      MOVE    #BOOT,R0                ; relocate boot loader (128 words)
            MOVE    #START,R1               ; new start location
            DO      #$40,END_RELOC          ;
            MOVEM   P:(R0)+,A               ;
            NOP                             ;
            MOVEM   A1,P:(R1)+              ;
            NOP                             ;
END_RELOC   JMP     START                   ;

            ORG     P:START,P:BOOT          ; P:runtime, P:loadtime
MAIN        CLR     B   #$0,A               ; clear checksum
    IF (VER==3)
            JSR     READ16                  ; get cmd | complement
            MOVE    A1,Y0                   ;
            MOVE    #>$FE01,A               ; check cmd
            CMP     Y0,A                    ;
            JNE     MAIN                    ;
    ENDIF
            JSR     READ16                  ; get NWORDS
            MOVE    A1,X1                   ; save word count
            JSR     READ16                  ; get MEMSPACE -- X,Y,P,END
            MOVE    A1,X0                   ; save memspace
            JSR     READ16                  ; get ADDRESS [15..0]
            MOVE    A1,R0                   ; save start address
    IF (VER==2)
            JSR     READ16                  ; get address [23..16], ignore
    ENDIF
            JSET    #$3,X0,FINISH           ; found END record

GET_WORDS   DO      X1,END_LOAD
            JSR     READ16                  ; get bits [15..0]
            ASR     #$10,A,A                ; shift right 16 bits
            JSR     READ16                  ; get bits [23..16]
            ASL     #$10,A,A                ; 24-bit word in A1
X_LOAD      JCLR    #$0,X0,Y_LOAD           ; check X flag
            MOVE    A1,X:(R0)+
Y_LOAD      JCLR    #$1,X0,P_LOAD           ; check Y flag
            MOVE    A1,Y:(R0)+
P_LOAD      JCLR    #$2,X0,NO_LOAD          ; check P flag
            MOVEM   A1,P:(R0)+
NO_LOAD     NOP
END_LOAD    NOP
    IF (VER==2)
ACK         JCLR    #$6,X:SSISR,*           ; wait for TDE to go high
            MOVE    B1,X:TXD                ; write checksum to ESSI
ENDWR       NOP                             ;
    ENDIF
            JMP     MAIN                    ; get next S-record

    IF (VER==2)
READ16      JCLR    #$7,X:SSISR,*           ; wait for RDF to go high
            MOVE    X:RXD,A1                ; read from ESSI
            MOVE    A,Y1                    ;
            ADD     Y1,B                    ; add to checksum (in B)
END_READ    RTS
    ENDIF

    IF (VER==3)
READ16      JCLR    #$4,X:PDRD,*            ; wait for data in RXREG
            MOVE    Y:RXREG,A1              ; bits 15..0
            AND     #>$FFFF,A               ;
            MOVE    A,Y1                    ; add to checksum (in B)
            ADD     Y1,B                    ;
ENDRD16     NOP                             ; 16-bit word in A1 (was RTS)

WRITE16     MOVE    A1,Y:TXREG              ; write bits 15..0
            REP     #$10                    ;
            NOP                             ;
ENDWR16     RTS                             ;
    ENDIF

FINISH      JMP     $0                      ;begin executing loaded program
